export { getVirtualFilePageConfigValuesAll };
import { assert, assertPosixPath } from '../../../utils.js';
import { generateEagerImport } from '../generateEagerImport.js';
import { getVirtualFileIdPageConfigValuesAll, isVirtualFileIdPageConfigValuesAll } from '../../../../shared/virtual-files/virtualFilePageConfigValuesAll.js';
import { getVikeConfig } from './getVikeConfig.js';
import { extractAssetsAddQuery } from '../../../../shared/extractAssetsQuery.js';
import { debug } from './debug.js';
import path from 'path';
import { getConfigValue } from '../../../../../shared/page-configs/utils.js';
import { getConfigValueSourcesRelevant } from '../../../shared/getConfigValueSource.js';
import { isConfigEnvMatch } from './isConfigEnvMatch.js';
async function getVirtualFilePageConfigValuesAll(id, userRootDir, isDev, configVps) {
    const result = isVirtualFileIdPageConfigValuesAll(id);
    assert(result);
    /* This assertion fails when using includeAssetsImportedByServer
    {
      const isForClientSide = !config.build.ssr
      assert(result.isForClientSide === isForClientSide)
    }
    */
    const { pageId, isForClientSide } = result;
    const { pageConfigs } = await getVikeConfig(userRootDir, isDev, configVps.extensions, true);
    const pageConfig = pageConfigs.find((pageConfig) => pageConfig.pageId === pageId);
    assert(pageConfig);
    const code = getLoadConfigValuesAll(pageConfig, isForClientSide, pageId, configVps.includeAssetsImportedByServer, isDev);
    debug(id, isForClientSide ? 'CLIENT-SIDE' : 'SERVER-SIDE', code);
    return code;
}
function getLoadConfigValuesAll(pageConfig, isForClientSide, pageId, includeAssetsImportedByServer, isDev) {
    const configValue = getConfigValue(pageConfig, 'clientRouting', 'boolean');
    const isClientRouting = configValue?.value ?? false;
    const lines = [];
    const importStatements = [];
    lines.push('export default [');
    let varCounter = 0;
    getConfigValueSourcesRelevant(pageConfig).forEach((configValueSource) => {
        const { valueIsImportedAtRuntime, configName, configEnv, definedAtInfo } = configValueSource;
        if (!valueIsImportedAtRuntime)
            return;
        if (configValueSource.valueIsFilePath)
            return;
        if (!isConfigEnvMatch(configEnv, isForClientSide, isClientRouting))
            return;
        const { filePath, fileExportPath } = definedAtInfo;
        assertPosixPath(filePath);
        const fileName = path.posix.basename(filePath);
        const isPlusFile = fileName.startsWith('+');
        const fileExportName = fileExportPath[0];
        assert(!configValueSource.valueIsFilePath);
        assert(fileExportName);
        const { importVar, importStatement } = generateEagerImport(filePath, varCounter++, isPlusFile ? undefined : fileExportName);
        importStatements.push(importStatement);
        lines.push(`  {`);
        lines.push(`    configName: '${configName}',`);
        lines.push(`    importFilePath: '${filePath}',`);
        lines.push(`    isPlusFile: ${JSON.stringify(isPlusFile)},`);
        if (isPlusFile) {
            lines.push(`    importFileExports: ${importVar},`);
        }
        else {
            lines.push(`    importFileExportValue: ${importVar},`);
            assert(fileExportName);
            lines.push(`    importFileExportName: ${JSON.stringify(fileExportName)},`);
        }
        lines.push(`  },`);
    });
    lines.push('];');
    if (includeAssetsImportedByServer && isForClientSide && !isDev) {
        lines.push(`import '${extractAssetsAddQuery(getVirtualFileIdPageConfigValuesAll(pageId, false))}'`);
    }
    const code = [...importStatements, ...lines].join('\n');
    return code;
}
