export { mergeScriptTags };
import { assert } from '../../utils.js';
const scriptRE = /(<script\b(?:\s[^>]*>|>))(.*?)<\/script>/gims;
const srcRE = /\bsrc\s*=\s*(?:"([^"]+)"|'([^']+)'|([^\s'">]+))/im;
const typeRE = /\btype\s*=\s*(?:"([^"]+)"|'([^']+)'|([^\s'">]+))/im;
function mergeScriptTags(scriptTagsHtml, isProduction) {
    let scriptTag = '';
    const scripts = parseScripts(scriptTagsHtml);
    // We need to merge module scripts to ensure execution order
    {
        const scriptsModule = scripts.filter(({ isModule }) => isModule);
        if (scriptsModule.length === 1) {
            scriptTag += scriptsModule[0].outerHtml;
        }
        else {
            const contents = [];
            scriptsModule.forEach(({ src, innerHtml }) => {
                const hasInnerHtml = !!innerHtml.trim();
                if (src) {
                    assert(!hasInnerHtml);
                    if (isProduction) {
                        contents.push(`import ${JSON.stringify(src)};`);
                    }
                    else {
                        // Ensure HMR preamble code is executed before client entries
                        contents.push(`import(${JSON.stringify(src)});`);
                    }
                }
                else if (hasInnerHtml) {
                    innerHtml = innerHtml.split('\n').filter(Boolean).join('\n');
                    contents.push(innerHtml);
                }
            });
            if (contents.length > 0) {
                scriptTag += `<script type="module" defer>\n${contents.join('\n')}\n</script>`;
            }
        }
    }
    // We don't need to merge ES5 scripts
    {
        const scriptsES5 = scripts.filter(({ isModule }) => !isModule);
        scriptsES5.forEach(({ outerHtml }) => {
            scriptTag += outerHtml;
        });
    }
    return scriptTag;
}
function parseScripts(htmlString) {
    const scripts = [];
    let match;
    while ((match = scriptRE.exec(htmlString))) {
        const [outerHtml, openTag, innerHtml] = match;
        assert(outerHtml && openTag && innerHtml !== undefined);
        let isModule = false;
        {
            const typeMatch = openTag.match(typeRE);
            const type = typeMatch && (typeMatch[1] || typeMatch[2] || typeMatch[3]);
            isModule = type === 'module';
        }
        let src = null;
        {
            const srcMatch = openTag.match(srcRE);
            src = (srcMatch && (srcMatch[1] || srcMatch[2] || srcMatch[3])) || '';
        }
        scripts.push({ isModule, src, innerHtml, outerHtml });
    }
    return scripts;
}
