export { assertPageContextProvidedByUser };
import { assert, assertUsage, assertWarning, isObject } from './utils.js';
import pc from '@brillout/picocolors';
function assertPageContextProvidedByUser(pageContextProvidedByUser, { hookName, hookFilePath }) {
    if (pageContextProvidedByUser === undefined || pageContextProvidedByUser === null)
        return;
    assert(!hookName.endsWith(')'));
    const errPrefix = `The ${pc.cyan('pageContext')} object provided by the ${hookName}() hook defined by ${hookFilePath}`;
    assertUsage(isObject(pageContextProvidedByUser), `${errPrefix} should be an object (but it's ${pc.cyan(`typeof pageContext === ${JSON.stringify(typeof pageContextProvidedByUser)}`)} instead)`);
    assertUsage(!('_objectCreatedByVitePluginSsr' in pageContextProvidedByUser), `${errPrefix} shouldn't be the whole ${pc.cyan('pageContext')} object, see https://vite-plugin-ssr.com/pageContext-manipulation#do-not-return-entire-pagecontext`);
    // In principle, it's possible to use onBeforeRoute()` to override and define the whole routing.
    // Is that a good idea to allow users to do this? Beyond deep integration with Vue Router or React Router, is there a use case for this?
    assertWarning(!('_pageId' in pageContextProvidedByUser), `${errPrefix} sets ${pc.cyan('pageContext._pageId')} which means that vite-plugin-ssr's routing is overriden. This is an experimental feature: make sure to contact a vite-plugin-ssr maintainer before using this.`, { onlyOnce: true });
    assertUsage(!('is404' in pageContextProvidedByUser), `${errPrefix} sets ${pc.cyan('pageContext.is404')} which is forbidden, use ${pc.cyan('throw render()')} instead, see https://vite-plugin-ssr.com/render`);
}
